#Required libraries.
library(ncdf4)
library(M3)
library(fields)
library(animation)

load("/work/ROMO/users/shn/o3rea_2020/model_indicatorratio/sensitivity.mapdata.2007.rdata")

deltaO3.gt70.catcol.2007 = deltaO3.gt70.catcol 
deltaO3.maysep.catcol.2007 = deltaO3.maysep.catcol 

rm(deltaO3.gt70.catcol, deltaO3.maysep.catcol)

load("/work/ROMO/users/shn/o3rea_2020/model_indicatorratio/sensitivity.mapdata.2016.rdata")

deltaO3.gt70.catcol.2016 = deltaO3.gt70.catcol 
deltaO3.maysep.catcol.2016 = deltaO3.maysep.catcol 

rm(deltaO3.gt70.catcol, deltaO3.maysep.catcol)


 region.df = data.frame(name = c("National"), lon.min = c(-122),lon.max =c(-69), lat.min = c(24), lat.max = c(52)) 


## loop through regions
for (r in c(1:nrow(region.df))) {

reg = as.character(region.df[r,"name"])


#> Location of R scripts for post processing and evaluation.
r.source <- "/work/romo/users/shn/Rfunctions/"

#> Set location where plots will be saved.
outdir <- "./" #"/work/ROMO/users/shn/o3rea_2020/model_indicatorratio/plots/annual_maps"
#set a common zlim for all species and all time steps in the cctm files.
#> If this is left blank (i.e. set.zlim <- "") or commented out, the zlim will be determined
#> for each species based on the min and max of the data across all time steps in the cctm file.
#set.zlim <- c(0,100)
set.zlim.ndays <- c(0,20)

#> Set a common zlim for the difference plots and relative difference plots.
#> These zlims will be the same for all species and all time steps.
#> If these zlims are left blank (e.g. set.diff.zlim <- "") or commented out, the zlim will be determined
#> for each species based on the min and max across all time steps of the difference between the two cctm files.
set.diff.zlim <- c(-5,5)
#set.rel.diff.zlim <- c(-100,100)


####################################################

#> Create a list of variables and units in  cctm.file1. TFLAG will always be the first variable listed in an I/O API file.
cctm1.spec <- unlist(lapply(cctm1.in$var, function(var)var$name))[-1]
#> Use gsub to strip out extra spaces.
cctm1.unit <- gsub("  ","",unlist(lapply(cctm1.in$var, function(var)var$units))[-1])

#> Pull out the time steps and the grid coordinates associated with the data.
#> These M3 functions are wrappers for functions in the ncdf4 package.
datetime <- get.datetime.seq(cctm.file1)
YYYYMM <- paste(substr(datetime[1],1,4),substr(datetime[1],6,7),sep="")
format.date.seq <- format.Date(datetime,"%m/%d/%Y")
#> Lambert projected coordinates (unit=km)
x.proj.coord.conus <- get.coord.for.dimension(cctm.file1,"col")$coords
y.proj.coord.conus <- get.coord.for.dimension(cctm.file1,"row")$coords
#> remove border cells for CMAQ-CAMx comparison
#> Map lon/lat sequence to projected coordinates (to annotate spatial maps).
#lonlat.range <- project.M3.to.lonlat(range(x.proj.coord),range(y.proj.coord),cctm.file1,"km")$coords
lon.range <- c(region.df[r,"lon.min"],region.df[r,"lon.max"])
lat.range <- c(region.df[r,"lat.min"],region.df[r,"lat.max"])
#lon.range <- floor(lonlat.range[,1])
#lat.range <- floor(lonlat.range[,2])
lon.seq <- seq(lon.range[1],lon.range[2]+1,by=2)
lat.seq <- seq(lat.range[1],lat.range[2]+1,by=2)
x.plot.seq <- project.lonlat.to.M3(lon.seq,rep(30,length=length(lon.seq)),cctm.file1,"km")$coords[,1]
y.plot.seq <- project.lonlat.to.M3(rep(-90,length=length(lat.seq)),lat.seq,cctm.file1,"km")$coords[,2]
#> US, Canada, Mexico map lines in Lambert projected coordinates.
map.lines <- get.map.lines.M3.proj(cctm.file1,database="canusamex")$coords
map.lines.co <- get.map.lines.M3.proj(cctm.file1,database="county")$coords
x.proj.coord = x.proj.coord.conus[which(x.proj.coord.conus < max(x.plot.seq) & x.proj.coord.conus > min(x.plot.seq))]
y.proj.coord = y.proj.coord.conus[which(y.proj.coord.conus < max(y.plot.seq) & y.proj.coord.conus > min(y.plot.seq))]


####################################################
#> Check the species list that was provided through the SPEC variable.
#> If "ALL" is used, will plot all the species in the file. 
#> Note that this check is not case sensitive (i.e. can use "all","All", etc).
#> If a list is provided, make sure the species names match the names in the cctm.file1

  
  plot.spec.list <- cctm1.spec 
  plot.unit.list <- cctm1.unit

  spec.name = as.character(plot.spec.list[1])
 
  #Get the units associated with the species list to be plotted.
  find.unit.fun <- function(x)cctm1.unit[which(cctm1.spec==x)]
  plot.unit.list <- gsub("  ","",unlist(lapply(as.list(plot.spec.list),find.unit.fun)))


####################################################
#> Source file containing color palettes (my.colors, my.col.cool, my.col.warm),
#> function for setting up zlim and color scale for concentration maps (find.zlim)
#> and function for setting up zlim and color scale for maps of differences or % differences (find.diff.zlim)
source(paste(r.source,"/find_zlim.R",sep=""))

####################################################
#> Check is zlim values have been provided.  
#> If they are missing, set them as NA to enable code to set 
#> them based on the distribution of the data in the cctm file. 
if(!exists("set.zlim")) set.zlim <- NA
if(!exists("set.diff.zlim")) set.diff.zlim <- NA
if(!exists("set.rel.diff.zlim")) set.rel.diff.zlim <- NA

####################################################
#> Check is second cctm file has been provided.
#> If missing, set cctm.file2 and appl2 to "" since
#> this is what value they would have if they were 
#> not set as environment values when using
#> a shell script to run this code.


####################################################
#> Set up size of jpeg plot. This is based of aspect_ratio_plot.R developed by J. Swall.
#> mai = a numerical vector of the form c(bottom, left, top, right) which gives the margin size specified in inches.
my.mai <- c(1.1, 0.4, 0.4, 1.5)
#> Width of jpeg plots (in inches). 
my.width <- 6 	
#> Determine height of plots based on the aspect ratio of the model domain.
prop.y.to.x <- (max(y.proj.coord)-min(y.proj.coord)) / (max(x.proj.coord)-min(x.proj.coord))
plot.width <- my.width - my.mai[2] - my.mai[4]
plot.height <- plot.width * prop.y.to.x
my.height <- plot.height + my.mai[1] + my.mai[3]

library("viridis")
# viridis
 vir = viridis(20)
# catcols = c("white",vir[4],vir[20], vir[12])
# catcols = c("white","red","orange","yellow")
 catcols = c("white","#20639B","#F6D55C","#ED553B")

 my.cat.at = c(0,1,2,3)
 my.cat.labels = c("No days","NOx lim","NOx sat","Mix")

# catcols = c(vir[4],vir[20], vir[12])
# my.cat.at = c(0,1,2)
# my.cat.labels = c("NOx lim","NOx sat","Mix")



 spec.unit = as.character(plot.unit.list[1])

 #subset arrays to regional domain
 deltaO3.gt70.catcol.2007.r <- deltaO3.gt70.catcol.2007[which(x.proj.coord.conus < max(x.plot.seq) & x.proj.coord.conus > min(x.plot.seq)),which(y.proj.coord.conus < max(y.plot.seq) & y.proj.coord.conus > min(y.plot.seq))]
 deltaO3.maysep.catcol.2007.r <- deltaO3.maysep.catcol.2007[which(x.proj.coord.conus < max(x.plot.seq) & x.proj.coord.conus > min(x.plot.seq)),which(y.proj.coord.conus < max(y.plot.seq) & y.proj.coord.conus > min(y.plot.seq))]
 deltaO3.gt70.catcol.2016.r <- deltaO3.gt70.catcol.2016[which(x.proj.coord.conus < max(x.plot.seq) & x.proj.coord.conus > min(x.plot.seq)),which(y.proj.coord.conus < max(y.plot.seq) & y.proj.coord.conus > min(y.plot.seq))]
 deltaO3.maysep.catcol.2016.r <- deltaO3.maysep.catcol.2016[which(x.proj.coord.conus < max(x.plot.seq) & x.proj.coord.conus > min(x.plot.seq)),which(y.proj.coord.conus < max(y.plot.seq) & y.proj.coord.conus > min(y.plot.seq))]



 #Set up zlim and color bar for concentration maps of cctm.file1 (new simulation) and cctm.file2 (base simulation).
 #This will be common for all days in the monthly file.
 #If this is not set by the user, it will be chosen based on the distribution of the data in the 2 cctm files. 
 if(length(set.zlim.ndays)==2){
   zlim.list <- find.zlim(c(set.zlim.ndays[1]:set.zlim.ndays[2]),max(set.zlim.ndays))
   }else{
   zlim.list <- find.zlim(c(spec.array.new,spec.array.base),50)}
 my.zlim <- zlim.list$my.zlim
 my.col <- zlim.list$my.col
 my.at <- zlim.list$my.at
 my.labels <- zlim.list$my.labels

 #Set up zlim and color bar for difference plots.
 if(length(set.diff.zlim)==2){
   diff.zlim.list <- find.diff.zlim(c(set.diff.zlim[1]:set.diff.zlim[2]),50)
   }else{
   diff.zlim.list <- find.diff.zlim(c(spec.array.diff),50)}
 my.diff.zlim <- diff.zlim.list$my.diff.zlim
 my.diff.col <- diff.zlim.list$my.diff.col
 my.diff.at <- diff.zlim.list$my.diff.at
 my.diff.labels <- diff.zlim.list$my.diff.labels

## plot O3 sensitivity categories for high days and May-Sep

   png(paste(outdir,"/",spec.name,"_O3sensitivity_categories_highdays.vs.maysep_4panel_",reg,"_2007.vs.2016.blue_yellow_orange.png",sep = ""),width = my.width*150, height = my.height*150, units = "px")
   par(mai=c(0.5, 0.4, 0.4, 0.5),oma=c(2,0,0,0),mfrow=c(2,2))

   #Map Delta MDA8: 20% NOx cut on days > 70 ppb in 2007
   plot(1,xlim=range(x.proj.coord),ylim=range(y.proj.coord),axes=F,xlab="",ylab="",type="n",main=paste0("Average ",spec.name," Chemical Sensitivity \n on days > 70 ppb in 2007"))
#   axis(1,at=x.plot.seq,labels=lon.seq)	#Add longitude to x axis.
#   axis(2,at=y.plot.seq,labels=lat.seq)	#Add latitude to y axis.
   image(x.proj.coord,y.proj.coord,deltaO3.gt70.catcol.2007.r,add=T,zlim=c(0,3),col=catcols)  #Add color image.
#   image.plot(legend.only=T,legend.args=list(text=paste(spec.unit)),axis.args=list(at=my.cat.at,labels=my.cat.labels),
#	    zlim=c(0,3),col=catcols,legend.width=1.4)  #Add legend bar.
   lines(map.lines)	#Add state lines.
   if(r %in% c(10:23)) {lines(map.lines.co, lwd = 0.5)}	#Add county lines for urban areas
   box()

   #Map Delta MDA8: 20% NOx cut on days > 70 ppb in 2016
   plot(1,xlim=range(x.proj.coord),ylim=range(y.proj.coord),axes=F,xlab="",ylab="",type="n",main=paste0("Average ",spec.name," Chemical Sensitivity \n on days > 70 ppb in 2016"))
#   axis(1,at=x.plot.seq,labels=lon.seq)	#Add longitude to x axis.
#   axis(2,at=y.plot.seq,labels=lat.seq)	#Add latitude to y axis.
   image(x.proj.coord,y.proj.coord,deltaO3.gt70.catcol.2016.r,add=T,zlim=c(0,3),col=catcols)  #Add color image.
#   image.plot(legend.only=T,legend.args=list(text=paste(spec.unit)),axis.args=list(at=my.cat.at,labels=my.cat.labels),
#	    zlim=c(0,3),col=catcols,legend.width=1.4)  #Add legend bar.
   lines(map.lines)	#Add state lines.
   if(r %in% c(10:23)) {lines(map.lines.co, lwd = 0.5)}	#Add county lines for urban areas
   box()

   #Map Delta MDA8: 20% NOx cut on May-Sep days in 2007
   plot(1,xlim=range(x.proj.coord),ylim=range(y.proj.coord),axes=F,xlab="",ylab="",type="n",main=paste0("Average ",spec.name," Chemical Sensitivity \n from May-Sep in 2007"))
#   axis(1,at=x.plot.seq,labels=lon.seq)	#Add longitude to x axis.
#   axis(2,at=y.plot.seq,labels=lat.seq)	#Add latitude to y axis.
   image(x.proj.coord,y.proj.coord,deltaO3.maysep.catcol.2007.r,add=T,zlim=c(0,3),col=catcols)  #Add color image.
#   image.plot(legend.only=T,legend.args=list(text=paste(spec.unit)),axis.args=list(at=my.cat.at,labels=my.cat.labels),
#	    zlim=c(0,3),col=catcols,legend.width=1.4)  #Add legend bar.
   lines(map.lines)	#Add state lines.
   if(r %in% c(10:23)) {lines(map.lines.co, lwd = 0.5)}	#Add county lines for urban areas
   box()

   #Map Delta MDA8: 20% NOx cut on May-Sep days in 2016
   plot(1,xlim=range(x.proj.coord),ylim=range(y.proj.coord),axes=F,xlab="",ylab="",type="n",main=paste0("Average ",spec.name," Chemical Sensitivity \n from May-Sep in 2016"))
#   axis(1,at=x.plot.seq,labels=lon.seq)	#Add longitude to x axis.
#   axis(2,at=y.plot.seq,labels=lat.seq)	#Add latitude to y axis.
   image(x.proj.coord,y.proj.coord,deltaO3.maysep.catcol.2016.r,add=T,zlim=c(0,3),col=catcols)  #Add color image.
#   image.plot(legend.only=T,legend.args=list(text=paste(spec.unit)),axis.args=list(at=my.cat.at,labels=my.cat.labels),
#	    zlim=c(0,3),col=catcols,legend.width=1.4)  #Add legend bar.
   lines(map.lines)	#Add state lines.
   if(r %in% c(10:23)) {lines(map.lines.co, lwd = 0.5)}	#Add county lines for urban areas
   box()

par(mfrow=c(1,1),mar=c(0,0,0,0),oma=c(0,20,12,20))
image.plot(legend.only=T,legend.args=list(text=paste(spec.unit)),horizontal=T,axis.args=list(at=my.cat.at,labels=my.cat.labels),
           zlim=c(0,3),col=catcols,legend.width=1.4,cex=0.6)  #Add legend bar.

   dev.off()




} # end loop on regions


#> Script completed without error. 
print("spatial_automate complete.")


